@@ -1,9 +1,9 @@
 from .base_agents import Agent
+from .base_agents import MultiObjectiveAgent
 from .bandit_agents import MultiArmedAgent
 from .bandit_agents import ContextualAgent
 from .bandit_agents import CombinatorialAgent
 from .bandit_agents import ContextualCombinatorialAgent
-from .bandit_agents import MultiObjectiveAgent
 from .bandit_agents import Random
 from .bandit_agents import TS
 from .bandit_agents import OGreedy
