@@ -1,3 +1,17 @@
+# Copyright 2021 Baihan Lin
+#
+# Licensed under the GNU General Public License, Version 3.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     https://www.gnu.org/licenses/gpl-3.0.en.html
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 """
@@ -41,8 +55,12 @@ def quantize_metrics(df, quantile_bin=False, nbins=20):
             df.loc[selected, "reward_bins"] = pd.cut(df[selected]["reward"], bins=nbins)
             df.loc[selected, "cost_bins"] = pd.cut(df[selected]["cost"], bins=nbins)
         else:
-            df.loc[selected, "reward_bins"] = pd.qcut(df[selected]["reward"], q=nbins)
-            df.loc[selected, "cost_bins"] = pd.qcut(df[selected]["cost"], q=nbins)
+            df.loc[selected, "reward_bins"] = pd.qcut(
+                df[selected]["reward"], q=nbins, duplicates="drop"
+            )
+            df.loc[selected, "cost_bins"] = pd.qcut(
+                df[selected]["cost"], q=nbins, duplicates="drop"
+            )
     df["reward_"] = df.groupby("reward_bins")["reward"].transform("mean")
     df["cost_"] = df.groupby("cost_bins")["cost"].transform("mean")
     return df
